@@ -6,26 +6,22 @@ import (
 	"errors"
 	"fmt"
 	"reflect"
-	"sort"
 	"strings"
 	"sync"
 	"time"
 
-	"k8s.io/apimachinery/pkg/runtime"
-
 	errorwrap "github.com/pkg/errors"
 	"github.com/sirupsen/logrus"
 	"google.golang.org/grpc/connectivity"
 	corev1 "k8s.io/api/core/v1"
 	rbacv1 "k8s.io/api/rbac/v1"
 	"k8s.io/apiextensions-apiserver/pkg/apis/apiextensions"
-	v1beta1ext "k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1beta1"
+	apiextensionsv1 "k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1"
+	apiextensionsv1beta1 "k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1beta1"
 	"k8s.io/apiextensions-apiserver/pkg/apiserver/validation"
 	extinf "k8s.io/apiextensions-apiserver/pkg/client/informers/externalversions"
-	k8serrors "k8s.io/apimachinery/pkg/api/errors"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/apis/meta/v1/unstructured"
-	"k8s.io/apimachinery/pkg/labels"
 	"k8s.io/apimachinery/pkg/runtime/schema"
 	utilclock "k8s.io/apimachinery/pkg/util/clock"
 	utilerrors "k8s.io/apimachinery/pkg/util/errors"
@@ -39,8 +35,8 @@ import (
 	"k8s.io/client-go/util/retry"
 	"k8s.io/client-go/util/workqueue"
 
-	"github.com/operator-framework/operator-lifecycle-manager/pkg/api/apis/operators/reference"
-	"github.com/operator-framework/operator-lifecycle-manager/pkg/api/apis/operators/v1alpha1"
+	"github.com/operator-framework/api/pkg/operators/reference"
+	"github.com/operator-framework/api/pkg/operators/v1alpha1"
 	"github.com/operator-framework/operator-lifecycle-manager/pkg/api/client/clientset/versioned"
 	"github.com/operator-framework/operator-lifecycle-manager/pkg/api/client/informers/externalversions"
 	"github.com/operator-framework/operator-lifecycle-manager/pkg/controller/bundle"
@@ -49,7 +45,6 @@ import (
 	"github.com/operator-framework/operator-lifecycle-manager/pkg/controller/registry/grpc"
 	"github.com/operator-framework/operator-lifecycle-manager/pkg/controller/registry/reconciler"
 	"github.com/operator-framework/operator-lifecycle-manager/pkg/controller/registry/resolver"
-	controllerclient "github.com/operator-framework/operator-lifecycle-manager/pkg/lib/controller-runtime/client"
 	index "github.com/operator-framework/operator-lifecycle-manager/pkg/lib/index"
 	"github.com/operator-framework/operator-lifecycle-manager/pkg/lib/operatorclient"
 	"github.com/operator-framework/operator-lifecycle-manager/pkg/lib/operatorlister"
@@ -58,7 +53,6 @@ import (
 	"github.com/operator-framework/operator-lifecycle-manager/pkg/lib/scoped"
 	sharedtime "github.com/operator-framework/operator-lifecycle-manager/pkg/lib/time"
 	"github.com/operator-framework/operator-lifecycle-manager/pkg/metrics"
-	"k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1beta1"
 )
 
 const (
@@ -66,14 +60,13 @@ const (
 	secretKind             = "Secret"
 	clusterRoleKind        = "ClusterRole"
 	clusterRoleBindingKind = "ClusterRoleBinding"
+	configMapKind          = "ConfigMap"
+	csvKind                = "ClusterServiceVersion"
 	serviceAccountKind     = "ServiceAccount"
 	serviceKind            = "Service"
 	roleKind               = "Role"
 	roleBindingKind        = "RoleBinding"
 	generatedByKey         = "olm.generated-by"
-	maxInstallPlanCount    = 5
-	maxDeletesPerSweep     = 5
-	RegistryFieldManager   = "olm.registry"
 )
 
 // Operator represents a Kubernetes operator that executes InstallPlans by
@@ -106,7 +99,7 @@ type Operator struct {
 type CatalogSourceSyncFunc func(logger *logrus.Entry, in *v1alpha1.CatalogSource) (out *v1alpha1.CatalogSource, continueSync bool, syncError error)
 
 // NewOperator creates a new Catalog Operator.
-func NewOperator(ctx context.Context, kubeconfigPath string, clock utilclock.Clock, logger *logrus.Logger, resync time.Duration, configmapRegistryImage, utilImage string, operatorNamespace string, scheme *runtime.Scheme) (*Operator, error) {
+func NewOperator(ctx context.Context, kubeconfigPath string, clock utilclock.Clock, logger *logrus.Logger, resync time.Duration, configmapRegistryImage, utilImage string, operatorNamespace string) (*Operator, error) {
 	resyncPeriod := queueinformer.ResyncWithJitter(resync, 0.2)
 	config, err := clientcmd.BuildConfigFromFlags("", kubeconfigPath)
 	if err != nil {
@@ -135,11 +128,6 @@ func NewOperator(ctx context.Context, kubeconfigPath string, clock utilclock.Clo
 	// Create an OperatorLister
 	lister := operatorlister.NewLister()
 
-	ssaClient, err := controllerclient.NewForConfig(config, scheme, RegistryFieldManager)
-	if err != nil {
-		return nil, err
-	}
-
 	// Allocate the new instance of an Operator.
 	op := &Operator{
 		Operator:                 queueOperator,
@@ -160,7 +148,7 @@ func NewOperator(ctx context.Context, kubeconfigPath string, clock utilclock.Clo
 		clientAttenuator:         scoped.NewClientAttenuator(logger, config, opClient, crClient, dynamicClient),
 	}
 	op.sources = grpc.NewSourceStore(logger, 10*time.Second, 10*time.Minute, op.syncSourceState)
-	op.reconciler = reconciler.NewRegistryReconcilerFactory(lister, opClient, configmapRegistryImage, op.now, ssaClient)
+	op.reconciler = reconciler.NewRegistryReconcilerFactory(lister, opClient, configmapRegistryImage, op.now)
 
 	// Wire OLM CR sharedIndexInformers
 	crInformerFactory := externalversions.NewSharedInformerFactoryWithOptions(op.client, resyncPeriod())
@@ -335,8 +323,8 @@ func NewOperator(ctx context.Context, kubeconfigPath string, clock utilclock.Clo
 	}
 
 	// Register CustomResourceDefinition QueueInformer
-	crdInformer := extinf.NewSharedInformerFactory(op.opClient.ApiextensionsV1beta1Interface(), resyncPeriod()).Apiextensions().V1beta1().CustomResourceDefinitions()
-	op.lister.APIExtensionsV1beta1().RegisterCustomResourceDefinitionLister(crdInformer.Lister())
+	crdInformer := extinf.NewSharedInformerFactory(op.opClient.ApiextensionsInterface(), resyncPeriod()).Apiextensions().V1().CustomResourceDefinitions()
+	op.lister.APIExtensionsV1().RegisterCustomResourceDefinitionLister(crdInformer.Lister())
 	crdQueueInformer, err := queueinformer.NewQueueInformer(
 		ctx,
 		queueinformer.WithLogger(op.logger),
@@ -429,7 +417,9 @@ func (o *Operator) requeueOwners(obj metav1.Object) {
 
 		if queueSet != nil {
 			logger.WithField("ref", owner).Trace("requeuing owner")
-			queueSet.Requeue(namespace, owner.Name)
+			if err := queueSet.Requeue(namespace, owner.Name); err != nil {
+				logger.Warn(err.Error())
+			}
 		}
 	}
 }
@@ -547,7 +537,7 @@ func (o *Operator) syncConfigMap(logger *logrus.Entry, in *v1alpha1.CatalogSourc
 	}
 
 	if wasOwned := ownerutil.EnsureOwner(configMap, in); !wasOwned {
-		configMap, err = o.opClient.KubernetesInterface().CoreV1().ConfigMaps(configMap.GetNamespace()).Update(configMap)
+		configMap, err = o.opClient.KubernetesInterface().CoreV1().ConfigMaps(configMap.GetNamespace()).Update(context.TODO(), configMap, metav1.UpdateOptions{})
 		if err != nil {
 			syncError = fmt.Errorf("unable to write owner onto catalog source configmap - %v", err)
 			out.SetError(v1alpha1.CatalogSourceConfigMapError, syncError)
@@ -729,7 +719,7 @@ func (o *Operator) syncCatalogSources(obj interface{}) (syncError error) {
 	}
 
 	updateStatusFunc := func(catsrc *v1alpha1.CatalogSource) error {
-		latest, err := o.client.OperatorsV1alpha1().CatalogSources(catsrc.GetNamespace()).Get(catsrc.GetName(), metav1.GetOptions{})
+		latest, err := o.client.OperatorsV1alpha1().CatalogSources(catsrc.GetNamespace()).Get(context.TODO(), catsrc.GetName(), metav1.GetOptions{})
 		if err != nil {
 			logger.Errorf("error getting catalogsource - %v", err)
 			return err
@@ -738,7 +728,7 @@ func (o *Operator) syncCatalogSources(obj interface{}) (syncError error) {
 		out := latest.DeepCopy()
 		out.Status = catsrc.Status
 
-		if _, err := o.client.OperatorsV1alpha1().CatalogSources(out.GetNamespace()).UpdateStatus(out); err != nil {
+		if _, err := o.client.OperatorsV1alpha1().CatalogSources(out.GetNamespace()).UpdateStatus(context.TODO(), out, metav1.UpdateOptions{}); err != nil {
 			logger.Errorf("error while setting catalogsource status condition - %v", err)
 			return err
 		}
@@ -758,6 +748,10 @@ func (o *Operator) syncCatalogSources(obj interface{}) (syncError error) {
 
 	out, syncError := syncFunc(in, chain)
 
+	if out == nil {
+		return
+	}
+
 	if equalFunc(&catsrc.Status, &out.Status) {
 		return
 	}
@@ -783,8 +777,6 @@ func (o *Operator) syncResolvingNamespace(obj interface{}) error {
 		"id":        queueinformer.NewLoopID(),
 	})
 
-	o.gcInstallPlans(logger, namespace)
-
 	// get the set of sources that should be used for resolution and best-effort get their connections working
 	logger.Debug("resolving sources")
 
@@ -854,17 +846,6 @@ func (o *Operator) syncResolvingNamespace(obj interface{}) error {
 	// create installplan if anything updated
 	if len(updatedSubs) > 0 {
 		logger.Debug("resolution caused subscription changes, creating installplan")
-		// Finish calculating max generation by checking the existing installplans
-		installPlans, err := o.listInstallPlans(namespace)
-		if err != nil {
-			return err
-		}
-		for _, ip := range installPlans {
-			if gen := ip.Spec.Generation; gen > maxGeneration {
-				maxGeneration = gen
-			}
-		}
-
 		// any subscription in the namespace with manual approval will force generated installplans to be manual
 		// TODO: this is an odd artifact of the older resolver, and will probably confuse users. approval mode could be on the operatorgroup?
 		installPlanApproval := v1alpha1.ApprovalAutomatic
@@ -928,7 +909,7 @@ func (o *Operator) ensureSubscriptionInstallPlanState(logger *logrus.Entry, sub
 		return sub, false, nil
 	}
 
-	ip, err := o.client.OperatorsV1alpha1().InstallPlans(sub.GetNamespace()).Get(ipName, metav1.GetOptions{})
+	ip, err := o.client.OperatorsV1alpha1().InstallPlans(sub.GetNamespace()).Get(context.TODO(), ipName, metav1.GetOptions{})
 	if err != nil {
 		logger.WithField("installplan", ipName).Warn("unable to get installplan from cache")
 		return nil, false, err
@@ -947,7 +928,7 @@ func (o *Operator) ensureSubscriptionInstallPlanState(logger *logrus.Entry, sub
 	out.Status.CurrentCSV = out.Spec.StartingCSV
 	out.Status.LastUpdated = o.now()
 
-	updated, err := o.client.OperatorsV1alpha1().Subscriptions(sub.GetNamespace()).UpdateStatus(out)
+	updated, err := o.client.OperatorsV1alpha1().Subscriptions(sub.GetNamespace()).UpdateStatus(context.TODO(), out, metav1.UpdateOptions{})
 	if err != nil {
 		return nil, false, err
 	}
@@ -960,7 +941,7 @@ func (o *Operator) ensureSubscriptionCSVState(logger *logrus.Entry, sub *v1alpha
 		return sub, false, nil
 	}
 
-	csv, err := o.client.OperatorsV1alpha1().ClusterServiceVersions(sub.GetNamespace()).Get(sub.Status.CurrentCSV, metav1.GetOptions{})
+	csv, err := o.client.OperatorsV1alpha1().ClusterServiceVersions(sub.GetNamespace()).Get(context.TODO(), sub.Status.CurrentCSV, metav1.GetOptions{})
 	out := sub.DeepCopy()
 	if err != nil {
 		logger.WithError(err).WithField("currentCSV", sub.Status.CurrentCSV).Debug("error fetching csv listed in subscription status")
@@ -970,9 +951,9 @@ func (o *Operator) ensureSubscriptionCSVState(logger *logrus.Entry, sub *v1alpha
 		if err := querier.Queryable(); err != nil {
 			return nil, false, err
 		}
-		bundle, _, _ := querier.FindReplacement(&csv.Spec.Version.Version, sub.Status.CurrentCSV, sub.Spec.Package, sub.Spec.Channel, resolver.CatalogKey{Name: sub.Spec.CatalogSource, Namespace: sub.Spec.CatalogSourceNamespace})
-		if bundle != nil {
-			o.logger.Tracef("replacement %s bundle found for current bundle %s", bundle.CsvName, sub.Status.CurrentCSV)
+		b, _, _ := querier.FindReplacement(&csv.Spec.Version.Version, sub.Status.CurrentCSV, sub.Spec.Package, sub.Spec.Channel, resolver.CatalogKey{Name: sub.Spec.CatalogSource, Namespace: sub.Spec.CatalogSourceNamespace})
+		if b != nil {
+			o.logger.Tracef("replacement %s bundle found for current bundle %s", b.CsvName, sub.Status.CurrentCSV)
 			out.Status.State = v1alpha1.SubscriptionStateUpgradeAvailable
 		} else {
 			out.Status.State = v1alpha1.SubscriptionStateAtLatest
@@ -988,7 +969,7 @@ func (o *Operator) ensureSubscriptionCSVState(logger *logrus.Entry, sub *v1alpha
 	out.Status.LastUpdated = o.now()
 
 	// Update Subscription with status of transition. Log errors if we can't write them to the status.
-	updatedSub, err := o.client.OperatorsV1alpha1().Subscriptions(out.GetNamespace()).UpdateStatus(out)
+	updatedSub, err := o.client.OperatorsV1alpha1().Subscriptions(out.GetNamespace()).UpdateStatus(context.TODO(), out, metav1.UpdateOptions{})
 	if err != nil {
 		logger.WithError(err).Info("error updating subscription status")
 		return nil, false, fmt.Errorf("error updating Subscription status: " + err.Error())
@@ -1021,13 +1002,13 @@ func (o *Operator) updateSubscriptionStatus(namespace string, gen int, subs []*v
 
 			update := func() error {
 				// Update the status of the latest revision
-				latest, err := o.client.OperatorsV1alpha1().Subscriptions(s.GetNamespace()).Get(s.GetName(), getOpts)
+				latest, err := o.client.OperatorsV1alpha1().Subscriptions(s.GetNamespace()).Get(context.TODO(), s.GetName(), getOpts)
 				if err != nil {
 					return err
 				}
 
 				latest.Status = s.Status
-				_, err = o.client.OperatorsV1alpha1().Subscriptions(namespace).UpdateStatus(latest)
+				_, err = o.client.OperatorsV1alpha1().Subscriptions(namespace).UpdateStatus(context.TODO(), latest, metav1.UpdateOptions{})
 
 				return err
 			}
@@ -1105,7 +1086,7 @@ func (o *Operator) createInstallPlan(namespace string, gen int, subs []*v1alpha1
 		ownerutil.AddNonBlockingOwner(ip, sub)
 	}
 
-	res, err := o.client.OperatorsV1alpha1().InstallPlans(namespace).Create(ip)
+	res, err := o.client.OperatorsV1alpha1().InstallPlans(namespace).Create(context.TODO(), ip, metav1.CreateOptions{})
 	if err != nil {
 		return nil, err
 	}
@@ -1116,7 +1097,7 @@ func (o *Operator) createInstallPlan(namespace string, gen int, subs []*v1alpha1
 		CatalogSources: catalogSources,
 		BundleLookups:  bundleLookups,
 	}
-	res, err = o.client.OperatorsV1alpha1().InstallPlans(namespace).UpdateStatus(res)
+	res, err = o.client.OperatorsV1alpha1().InstallPlans(namespace).UpdateStatus(context.TODO(), res, metav1.UpdateOptions{})
 	if err != nil {
 		return nil, err
 	}
@@ -1199,77 +1180,6 @@ func (o *Operator) unpackBundles(plan *v1alpha1.InstallPlan) (bool, *v1alpha1.In
 	return unpacked, out, nil
 }
 
-// gcInstallPlans garbage collects installplans that are too old
-// installplans are ownerrefd to all subscription inputs, so they will not otherwise
-// be GCd unless all inputs have been deleted.
-func (o *Operator) gcInstallPlans(log logrus.FieldLogger, namespace string) {
-	allIps, err := o.lister.OperatorsV1alpha1().InstallPlanLister().InstallPlans(namespace).List(labels.Everything())
-	if err != nil {
-		log.Warn("unable to list installplans for GC")
-	}
-
-	if len(allIps) <= maxInstallPlanCount {
-		return
-	}
-
-	// we only consider maxDeletesPerSweep more than the allowed number of installplans for delete at one time
-	ips := allIps
-	if len(ips) > maxInstallPlanCount+maxDeletesPerSweep {
-		ips = allIps[:maxInstallPlanCount+maxDeletesPerSweep]
-	}
-
-	byGen := map[int][]*v1alpha1.InstallPlan{}
-	for _, ip := range ips {
-		gen, ok := byGen[ip.Spec.Generation]
-		if !ok {
-			gen = make([]*v1alpha1.InstallPlan, 0)
-		}
-		byGen[ip.Spec.Generation] = append(gen, ip)
-	}
-
-	gens := make([]int, 0)
-	for i := range byGen {
-		gens = append(gens, i)
-	}
-
-	sort.Ints(gens)
-
-	toDelete := make([]*v1alpha1.InstallPlan, 0)
-
-	for _, i := range gens {
-		g := byGen[i]
-
-		if len(ips)-len(toDelete) <= maxInstallPlanCount {
-			break
-		}
-
-		// if removing all installplans at this generation doesn't dip below the max, safe to delete all of them
-		if len(ips)-len(toDelete)-len(g) >= maxInstallPlanCount {
-			toDelete = append(toDelete, g...)
-			continue
-		}
-
-		// CreationTimestamp sorting shouldn't ever be hit unless there is a bug that causes installplans to be
-		// generated without bumping the generation. It is here as a safeguard only.
-
-		// sort by creation time
-		sort.Slice(g, func(i, j int) bool {
-			if !g[i].CreationTimestamp.Equal(&g[j].CreationTimestamp) {
-				return g[i].CreationTimestamp.Before(&g[j].CreationTimestamp)
-			}
-			// final fallback to lexicographic sort, in case many installplans are created with the same timestamp
-			return g[i].GetName() < g[j].GetName()
-		})
-		toDelete = append(toDelete, g[:len(ips)-len(toDelete)-maxInstallPlanCount]...)
-	}
-
-	for _, i := range toDelete {
-		if err := o.client.OperatorsV1alpha1().InstallPlans(namespace).Delete(i.GetName(), &metav1.DeleteOptions{}); err != nil {
-			log.WithField("deleting", i.GetName()).WithError(err).Warn("error GCing old installplan - may have already been deleted")
-		}
-	}
-}
-
 func (o *Operator) syncInstallPlans(obj interface{}) (syncError error) {
 	plan, ok := obj.(*v1alpha1.InstallPlan)
 	if !ok {
@@ -1291,6 +1201,28 @@ func (o *Operator) syncInstallPlans(obj interface{}) (syncError error) {
 		return
 	}
 
+	querier := o.serviceAccountQuerier.NamespaceQuerier(plan.GetNamespace())
+	ref, err := querier()
+	if err != nil {
+		syncError = fmt.Errorf("attenuated service account query failed - %v", err)
+		return
+	}
+
+	if ref != nil {
+		out := plan.DeepCopy()
+		out.Status.AttenuatedServiceAccountRef = ref
+
+		if !reflect.DeepEqual(plan, out) {
+			if _, updateErr := o.client.OperatorsV1alpha1().InstallPlans(out.GetNamespace()).UpdateStatus(context.TODO(), out, metav1.UpdateOptions{}); updateErr != nil {
+				syncError = fmt.Errorf("failed to attach attenuated ServiceAccount to status - %v", updateErr)
+				return
+			}
+
+			logger.WithField("attenuated-sa", ref.Name).Info("successfully attached attenuated ServiceAccount to status")
+			return
+		}
+	}
+
 	// Attempt to unpack bundles before installing
 	// Note: This should probably use the attenuated client to prevent users from resolving resources they otherwise don't have access to.
 	if len(plan.Status.BundleLookups) > 0 {
@@ -1302,7 +1234,7 @@ func (o *Operator) syncInstallPlans(obj interface{}) (syncError error) {
 
 		if !reflect.DeepEqual(plan.Status, out.Status) {
 			logger.Warnf("status not equal, updating...")
-			if _, err := o.client.OperatorsV1alpha1().InstallPlans(out.GetNamespace()).UpdateStatus(out); err != nil {
+			if _, err := o.client.OperatorsV1alpha1().InstallPlans(out.GetNamespace()).UpdateStatus(context.TODO(), out, metav1.UpdateOptions{}); err != nil {
 				syncError = fmt.Errorf("failed to update installplan bundle lookups: %v", err)
 			}
 
@@ -1322,28 +1254,6 @@ func (o *Operator) syncInstallPlans(obj interface{}) (syncError error) {
 		}
 	}
 
-	querier := o.serviceAccountQuerier.NamespaceQuerier(plan.GetNamespace())
-	reference, err := querier()
-	if err != nil {
-		syncError = fmt.Errorf("attenuated service account query failed - %v", err)
-		return
-	}
-
-	if reference != nil {
-		out := plan.DeepCopy()
-		out.Status.AttenuatedServiceAccountRef = reference
-
-		if !reflect.DeepEqual(plan, out) {
-			if _, updateErr := o.client.OperatorsV1alpha1().InstallPlans(out.GetNamespace()).UpdateStatus(out); err != nil {
-				syncError = fmt.Errorf("failed to attach attenuated ServiceAccount to status - %v", updateErr)
-				return
-			}
-
-			logger.WithField("attenuated-sa", reference.Name).Info("successfully attached attenuated ServiceAccount to status")
-			return
-		}
-	}
-
 	outInstallPlan, syncError := transitionInstallPlanState(logger.Logger, o, *plan, o.now())
 
 	if syncError != nil {
@@ -1369,7 +1279,7 @@ func (o *Operator) syncInstallPlans(obj interface{}) (syncError error) {
 	}()
 
 	// Update InstallPlan with status of transition. Log errors if we can't write them to the status.
-	if _, err := o.client.OperatorsV1alpha1().InstallPlans(plan.GetNamespace()).UpdateStatus(outInstallPlan); err != nil {
+	if _, err := o.client.OperatorsV1alpha1().InstallPlans(plan.GetNamespace()).UpdateStatus(context.TODO(), outInstallPlan, metav1.UpdateOptions{}); err != nil {
 		logger = logger.WithField("updateError", err.Error())
 		updateErr := errors.New("error updating InstallPlan status: " + err.Error())
 		if syncError == nil {
@@ -1427,55 +1337,54 @@ func (o *Operator) ResolvePlan(plan *v1alpha1.InstallPlan) error {
 	return nil
 }
 
-func getCRDVersionsMap(crd *v1beta1ext.CustomResourceDefinition) map[string]struct{} {
-	versionsMap := map[string]struct{}{}
+// Validate all existing served versions against new CRD's validation (if changed)
+func validateV1CRDCompatibility(dynamicClient dynamic.Interface, oldCRD *apiextensionsv1.CustomResourceDefinition, newCRD *apiextensionsv1.CustomResourceDefinition) error {
+	logrus.Debugf("Comparing %#v to %#v", oldCRD.Spec.Versions, newCRD.Spec.Versions)
 
-	for _, version := range crd.Spec.Versions {
-		versionsMap[version.Name] = struct{}{}
-	}
-	if crd.Spec.Version != "" {
-		versionsMap[crd.Spec.Version] = struct{}{}
+	// If validation schema is unchanged, return right away
+	newestSchema := newCRD.Spec.Versions[len(newCRD.Spec.Versions)-1].Schema
+	for i, oldVersion := range oldCRD.Spec.Versions {
+		if !reflect.DeepEqual(oldVersion.Schema, newestSchema) {
+			break
+		}
+		if i == len(oldCRD.Spec.Versions)-1 {
+			// we are on the last iteration
+			// schema has not changed between versions at this point.
+			return nil
+		}
 	}
 
-	return versionsMap
-}
-
-// Ensure all existing served versions are present in new CRD
-func ensureCRDVersions(oldCRD *v1beta1ext.CustomResourceDefinition, newCRD *v1beta1ext.CustomResourceDefinition) error {
-	newCRDVersions := getCRDVersionsMap(newCRD)
-
-	for _, oldVersion := range oldCRD.Spec.Versions {
-		if oldVersion.Served {
-			_, ok := newCRDVersions[oldVersion.Name]
-			if !ok {
-				return fmt.Errorf("New CRD (%s) must contain existing served versions (%s)", oldCRD.Name, oldVersion.Name)
+	convertedCRD := &apiextensions.CustomResourceDefinition{}
+	if err := apiextensionsv1.Convert_v1_CustomResourceDefinition_To_apiextensions_CustomResourceDefinition(newCRD, convertedCRD, nil); err != nil {
+		return err
+	}
+	for _, version := range oldCRD.Spec.Versions {
+		if !version.Served {
+			gvr := schema.GroupVersionResource{Group: oldCRD.Spec.Group, Version: version.Name, Resource: oldCRD.Spec.Names.Plural}
+			err := validateExistingCRs(dynamicClient, gvr, convertedCRD)
+			if err != nil {
+				return err
 			}
 		}
 	}
-	if oldCRD.Spec.Version != "" {
-		_, ok := newCRDVersions[oldCRD.Spec.Version]
-		if !ok {
-			return fmt.Errorf("New CRD (%s) must contain existing version (%s)", oldCRD.Name, oldCRD.Spec.Version)
-		}
-	}
+
+	logrus.Debugf("Successfully validated CRD %s\n", newCRD.Name)
 	return nil
 }
 
 // Validate all existing served versions against new CRD's validation (if changed)
-func (o *Operator) validateCustomResourceDefinition(oldCRD *v1beta1ext.CustomResourceDefinition, newCRD *v1beta1ext.CustomResourceDefinition) error {
-	o.logger.Debugf("Comparing %#v to %#v", oldCRD.Spec.Validation, newCRD.Spec.Validation)
-	// If validation schema is unchanged, return right away
-	if reflect.DeepEqual(oldCRD.Spec.Validation, newCRD.Spec.Validation) {
-		return nil
-	}
+func validateV1Beta1CRDCompatibility(dynamicClient dynamic.Interface, oldCRD *apiextensionsv1beta1.CustomResourceDefinition, newCRD *apiextensionsv1beta1.CustomResourceDefinition) error {
+	logrus.Debugf("Comparing %#v to %#v", oldCRD.Spec.Validation, newCRD.Spec.Validation)
+
+	// TODO return early of all versions are equal
 	convertedCRD := &apiextensions.CustomResourceDefinition{}
-	if err := v1beta1ext.Convert_v1beta1_CustomResourceDefinition_To_apiextensions_CustomResourceDefinition(newCRD, convertedCRD, nil); err != nil {
+	if err := apiextensionsv1beta1.Convert_v1beta1_CustomResourceDefinition_To_apiextensions_CustomResourceDefinition(newCRD, convertedCRD, nil); err != nil {
 		return err
 	}
 	for _, version := range oldCRD.Spec.Versions {
-		if version.Served {
+		if !version.Served {
 			gvr := schema.GroupVersionResource{Group: oldCRD.Spec.Group, Version: version.Name, Resource: oldCRD.Spec.Names.Plural}
-			err := o.validateExistingCRs(gvr, convertedCRD)
+			err := validateExistingCRs(dynamicClient, gvr, convertedCRD)
 			if err != nil {
 				return err
 			}
@@ -1484,17 +1393,18 @@ func (o *Operator) validateCustomResourceDefinition(oldCRD *v1beta1ext.CustomRes
 
 	if oldCRD.Spec.Version != "" {
 		gvr := schema.GroupVersionResource{Group: oldCRD.Spec.Group, Version: oldCRD.Spec.Version, Resource: oldCRD.Spec.Names.Plural}
-		err := o.validateExistingCRs(gvr, convertedCRD)
+		err := validateExistingCRs(dynamicClient, gvr, convertedCRD)
 		if err != nil {
 			return err
 		}
 	}
-	o.logger.Debugf("Successfully validated CRD %s\n", newCRD.Name)
+	logrus.Debugf("Successfully validated CRD %s\n", newCRD.Name)
 	return nil
 }
 
-func (o *Operator) validateExistingCRs(gvr schema.GroupVersionResource, newCRD *apiextensions.CustomResourceDefinition) error {
-	crList, err := o.dynamicClient.Resource(gvr).List(metav1.ListOptions{})
+func validateExistingCRs(dynamicClient dynamic.Interface, gvr schema.GroupVersionResource, newCRD *apiextensions.CustomResourceDefinition) error {
+	// make dynamic client
+	crList, err := dynamicClient.Resource(gvr).List(context.TODO(), metav1.ListOptions{})
 	if err != nil {
 		return fmt.Errorf("error listing resources in GroupVersionResource %#v: %s", gvr, err)
 	}
@@ -1511,33 +1421,6 @@ func (o *Operator) validateExistingCRs(gvr schema.GroupVersionResource, newCRD *
 	return nil
 }
 
-// Attempt to remove stored versions that have been deprecated before allowing
-// those versions to be removed from the new CRD.
-// The function may not always succeed as storedVersions requires at least one
-// version. If there is only stored version, it won't be removed until a new
-// stored version is added.
-func removeDeprecatedStoredVersions(oldCRD *v1beta1ext.CustomResourceDefinition, newCRD *v1beta1ext.CustomResourceDefinition) []string {
-	// StoredVersions requires to have at least one version.
-	if len(oldCRD.Status.StoredVersions) <= 1 {
-		return nil
-	}
-
-	newStoredVersions := []string{}
-	newCRDVersions := getCRDVersionsMap(newCRD)
-	for _, v := range oldCRD.Status.StoredVersions {
-		_, ok := newCRDVersions[v]
-		if ok {
-			newStoredVersions = append(newStoredVersions, v)
-		}
-	}
-
-	if len(newStoredVersions) < 1 {
-		return nil
-	} else {
-		return newStoredVersions
-	}
-}
-
 // ExecutePlan applies a planned InstallPlan to a namespace.
 func (o *Operator) ExecutePlan(plan *v1alpha1.InstallPlan) error {
 	if plan.Status.Phase != v1alpha1.InstallPlanPhaseInstalling {
@@ -1565,43 +1448,32 @@ func (o *Operator) ExecutePlan(plan *v1alpha1.InstallPlan) error {
 
 	ensurer := newStepEnsurer(kubeclient, crclient, dynamicClient)
 	r := newManifestResolver(plan.GetNamespace(), o.lister.CoreV1().ConfigMapLister(), o.logger)
+	b := newBuilder(kubeclient, dynamicClient, o.csvProvidedAPIsIndexer, r, o.logger)
 
 	for i, step := range plan.Status.Plan {
-		switch step.Status {
-		case v1alpha1.StepStatusPresent, v1alpha1.StepStatusCreated:
-			continue
-		case v1alpha1.StepStatusWaitingForAPI:
-			switch step.Resource.Kind {
-			case crdKind:
-				crd, err := o.opClient.ApiextensionsV1beta1Interface().ApiextensionsV1beta1().CustomResourceDefinitions().Get(step.Resource.Name, metav1.GetOptions{})
-				if err != nil {
-					if k8serrors.IsNotFound(err) {
-						plan.Status.Plan[i].Status = v1alpha1.StepStatusNotPresent
-					} else {
-						return errorwrap.Wrapf(err, "error finding the %s CRD", crd.Name)
-					}
-					continue
-				}
-
-				established, namesAccepted := false, false
-				for _, cdt := range crd.Status.Conditions {
-					switch cdt.Type {
-					case v1beta1.Established:
-						if cdt.Status == v1beta1.ConditionTrue {
-							established = true
-						}
-					case v1beta1.NamesAccepted:
-						if cdt.Status == v1beta1.ConditionTrue {
-							namesAccepted = true
-						}
-					}
-				}
-
-				if established && namesAccepted {
-					plan.Status.Plan[i].Status = v1alpha1.StepStatusCreated
-				}
-				continue
+		doStep := true
+		s, err := b.create(step)
+		if err != nil {
+			if _, ok := err.(notSupportedStepperErr); ok {
+				// stepper not implemented for this type yet
+				// stepper currently only implemented for CRD types
+				doStep = false
+			} else {
+				return err
 			}
+		}
+		if doStep {
+			status, err := s.Status()
+			if err != nil {
+				return err
+			}
+			plan.Status.Plan[i].Status = status
+			continue
+		}
+
+		switch step.Status {
+		case v1alpha1.StepStatusPresent, v1alpha1.StepStatusCreated, v1alpha1.StepStatusWaitingForAPI:
+			continue
 		case v1alpha1.StepStatusUnknown, v1alpha1.StepStatusNotPresent:
 			manifest, err := r.ManifestForStep(step)
 			if err != nil {
@@ -1609,73 +1481,6 @@ func (o *Operator) ExecutePlan(plan *v1alpha1.InstallPlan) error {
 			}
 			o.logger.WithFields(logrus.Fields{"kind": step.Resource.Kind, "name": step.Resource.Name}).Debug("execute resource")
 			switch step.Resource.Kind {
-			case crdKind:
-				// Marshal the manifest into a CRD instance.
-				var crd v1beta1ext.CustomResourceDefinition
-				err := json.Unmarshal([]byte(manifest), &crd)
-				if err != nil {
-					return errorwrap.Wrapf(err, "error parsing step manifest: %s", step.Resource.Name)
-				}
-
-				// TODO: check that names are accepted
-				// Attempt to create the CRD.
-				_, err = o.opClient.ApiextensionsV1beta1Interface().ApiextensionsV1beta1().CustomResourceDefinitions().Create(&crd)
-				if k8serrors.IsAlreadyExists(err) {
-					currentCRD, _ := o.lister.APIExtensionsV1beta1().CustomResourceDefinitionLister().Get(crd.GetName())
-					// Compare 2 CRDs to see if it needs to be updatetd
-					if !(reflect.DeepEqual(crd.Spec.Version, currentCRD.Spec.Version) &&
-						reflect.DeepEqual(crd.Spec.Versions, currentCRD.Spec.Versions) &&
-						reflect.DeepEqual(crd.Spec.Validation, currentCRD.Spec.Validation)) {
-						// Verify CRD ownership, only attempt to update if
-						// CRD has only one owner
-						// Example: provided=database.coreos.com/v1alpha1/EtcdCluster
-						matchedCSV, err := index.CRDProviderNames(o.csvProvidedAPIsIndexer, crd)
-						if err != nil {
-							return errorwrap.Wrapf(err, "error find matched CSV: %s", step.Resource.Name)
-						}
-						crd.SetResourceVersion(currentCRD.GetResourceVersion())
-						if len(matchedCSV) == 1 {
-							o.logger.Debugf("Found one owner for CRD %v", crd)
-						} else if len(matchedCSV) > 1 {
-							o.logger.Debugf("Found multiple owners for CRD %v", crd)
-
-							err := ensureCRDVersions(currentCRD, &crd)
-							if err != nil {
-								return errorwrap.Wrapf(err, "error missing existing CRD version(s) in new CRD: %s", step.Resource.Name)
-							}
-
-							if err = o.validateCustomResourceDefinition(currentCRD, &crd); err != nil {
-								return errorwrap.Wrapf(err, "error validating existing CRs agains new CRD's schema: %s", step.Resource.Name)
-							}
-						}
-						// Remove deprecated version in CRD storedVersions
-						storeVersions := removeDeprecatedStoredVersions(currentCRD, &crd)
-						if storeVersions != nil {
-							currentCRD.Status.StoredVersions = storeVersions
-							resultCRD, err := o.opClient.ApiextensionsV1beta1Interface().ApiextensionsV1beta1().CustomResourceDefinitions().UpdateStatus(currentCRD)
-							if err != nil {
-								return errorwrap.Wrapf(err, "error updating CRD's status: %s", step.Resource.Name)
-							}
-							crd.SetResourceVersion(resultCRD.GetResourceVersion())
-						}
-						// Update CRD to new version
-						_, err = o.opClient.ApiextensionsV1beta1Interface().ApiextensionsV1beta1().CustomResourceDefinitions().Update(&crd)
-						if err != nil {
-							return errorwrap.Wrapf(err, "error updating CRD: %s", step.Resource.Name)
-						}
-					}
-					// If it already existed, mark the step as Present.
-					plan.Status.Plan[i].Status = v1alpha1.StepStatusPresent
-					continue
-				} else if err != nil {
-					// Unexpected error creating the CRD.
-					return err
-				} else {
-					// If no error occured, make sure to wait for the API to become available.
-					plan.Status.Plan[i].Status = v1alpha1.StepStatusWaitingForAPI
-					continue
-				}
-
 			case v1alpha1.ClusterServiceVersionKind:
 				// Marshal the manifest into a CSV instance.
 				var csv v1alpha1.ClusterServiceVersion
@@ -1734,6 +1539,36 @@ func (o *Operator) ExecutePlan(plan *v1alpha1.InstallPlan) error {
 
 				plan.Status.Plan[i].Status = status
 
+			case resolver.BundleSecretKind:
+				var s corev1.Secret
+				err := json.Unmarshal([]byte(manifest), &s)
+				if err != nil {
+					return errorwrap.Wrapf(err, "error parsing step manifest: %s", step.Resource.Name)
+				}
+
+				// add ownerrefs on the secret that point to the CSV in the bundle
+				if step.Resolving != "" {
+					owner := &v1alpha1.ClusterServiceVersion{}
+					owner.SetNamespace(plan.GetNamespace())
+					owner.SetName(step.Resolving)
+					ownerutil.AddNonBlockingOwner(&s, owner)
+				}
+
+				// Update UIDs on all CSV OwnerReferences
+				updated, err := o.getUpdatedOwnerReferences(s.OwnerReferences, plan.Namespace)
+				if err != nil {
+					return errorwrap.Wrapf(err, "error generating ownerrefs for secret %s", s.GetName())
+				}
+				s.SetOwnerReferences(updated)
+				s.SetNamespace(namespace)
+
+				status, err := ensurer.EnsureBundleSecret(plan.Namespace, &s)
+				if err != nil {
+					return err
+				}
+
+				plan.Status.Plan[i].Status = status
+
 			case secretKind:
 				status, err := ensurer.EnsureSecret(o.namespace, plan.GetNamespace(), step.Resource.Name)
 				if err != nil {
@@ -1849,6 +1684,14 @@ func (o *Operator) ExecutePlan(plan *v1alpha1.InstallPlan) error {
 					return errorwrap.Wrapf(err, "error parsing step manifest: %s", step.Resource.Name)
 				}
 
+				// add ownerrefs on the service that point to the CSV in the bundle
+				if step.Resolving != "" {
+					owner := &v1alpha1.ClusterServiceVersion{}
+					owner.SetNamespace(plan.GetNamespace())
+					owner.SetName(step.Resolving)
+					ownerutil.AddNonBlockingOwner(&s, owner)
+				}
+
 				// Update UIDs on all CSV OwnerReferences
 				updated, err := o.getUpdatedOwnerReferences(s.OwnerReferences, plan.Namespace)
 				if err != nil {
@@ -1864,6 +1707,36 @@ func (o *Operator) ExecutePlan(plan *v1alpha1.InstallPlan) error {
 
 				plan.Status.Plan[i].Status = status
 
+			case configMapKind:
+				var cfg corev1.ConfigMap
+				err := json.Unmarshal([]byte(manifest), &cfg)
+				if err != nil {
+					return errorwrap.Wrapf(err, "error parsing step manifest: %s", step.Resource.Name)
+				}
+
+				// add ownerrefs on the configmap that point to the CSV in the bundle
+				if step.Resolving != "" {
+					owner := &v1alpha1.ClusterServiceVersion{}
+					owner.SetNamespace(plan.GetNamespace())
+					owner.SetName(step.Resolving)
+					ownerutil.AddNonBlockingOwner(&cfg, owner)
+				}
+
+				// Update UIDs on all CSV OwnerReferences
+				updated, err := o.getUpdatedOwnerReferences(cfg.OwnerReferences, plan.Namespace)
+				if err != nil {
+					return errorwrap.Wrapf(err, "error generating ownerrefs for configmap: %s", cfg.GetName())
+				}
+				cfg.SetOwnerReferences(updated)
+				cfg.SetNamespace(namespace)
+
+				status, err := ensurer.EnsureConfigMap(plan.Namespace, &cfg)
+				if err != nil {
+					return err
+				}
+
+				plan.Status.Plan[i].Status = status
+
 			default:
 				if !isSupported(step.Resource.Kind) {
 					// Not a supported resource
@@ -1953,7 +1826,7 @@ func (o *Operator) ExecutePlan(plan *v1alpha1.InstallPlan) error {
 // getExistingApiOwners creates a map of CRD names to existing owner CSVs in the given namespace
 func (o *Operator) getExistingApiOwners(namespace string) (map[string][]string, error) {
 	// Get a list of CSVs in the namespace
-	csvList, err := o.client.OperatorsV1alpha1().ClusterServiceVersions(namespace).List(metav1.ListOptions{})
+	csvList, err := o.client.OperatorsV1alpha1().ClusterServiceVersions(namespace).List(context.TODO(), metav1.ListOptions{})
 
 	if err != nil {
 		return nil, err
@@ -1978,7 +1851,7 @@ func (o *Operator) getUpdatedOwnerReferences(refs []metav1.OwnerReference, names
 
 	for i, owner := range refs {
 		if owner.Kind == v1alpha1.ClusterServiceVersionKind {
-			csv, err := o.client.OperatorsV1alpha1().ClusterServiceVersions(namespace).Get(owner.Name, metav1.GetOptions{})
+			csv, err := o.client.OperatorsV1alpha1().ClusterServiceVersions(namespace).Get(context.TODO(), owner.Name, metav1.GetOptions{})
 			if err != nil {
 				return nil, err
 			}
@@ -1990,7 +1863,7 @@ func (o *Operator) getUpdatedOwnerReferences(refs []metav1.OwnerReference, names
 }
 
 func (o *Operator) listSubscriptions(namespace string) (subs []*v1alpha1.Subscription, err error) {
-	list, err := o.client.OperatorsV1alpha1().Subscriptions(namespace).List(metav1.ListOptions{})
+	list, err := o.client.OperatorsV1alpha1().Subscriptions(namespace).List(context.TODO(), metav1.ListOptions{})
 	if err != nil {
 		return
 	}
@@ -2004,7 +1877,7 @@ func (o *Operator) listSubscriptions(namespace string) (subs []*v1alpha1.Subscri
 }
 
 func (o *Operator) listInstallPlans(namespace string) (ips []*v1alpha1.InstallPlan, err error) {
-	list, err := o.client.OperatorsV1alpha1().InstallPlans(namespace).List(metav1.ListOptions{})
+	list, err := o.client.OperatorsV1alpha1().InstallPlans(namespace).List(context.TODO(), metav1.ListOptions{})
 	if err != nil {
 		return
 	}
