@@ -6,11 +6,8 @@ import (
 	"errors"
 	"fmt"
 	"io/ioutil"
-	"math/rand"
-	"reflect"
 	"strings"
 	"testing"
-	"testing/quick"
 	"time"
 
 	"github.com/sirupsen/logrus"
@@ -20,7 +17,8 @@ import (
 	appsv1 "k8s.io/api/apps/v1"
 	corev1 "k8s.io/api/core/v1"
 	rbacv1 "k8s.io/api/rbac/v1"
-	"k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1beta1"
+	apiextensionsv1 "k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1"
+	apiextensionsv1beta1 "k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1beta1"
 	apiextensionsfake "k8s.io/apiextensions-apiserver/pkg/client/clientset/clientset/fake"
 	"k8s.io/apimachinery/pkg/api/meta"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
@@ -30,7 +28,6 @@ import (
 	"k8s.io/apimachinery/pkg/types"
 	utilclock "k8s.io/apimachinery/pkg/util/clock"
 	utilyaml "k8s.io/apimachinery/pkg/util/yaml"
-	"k8s.io/apiserver/pkg/storage/names"
 	fakedynamic "k8s.io/client-go/dynamic/fake"
 	"k8s.io/client-go/informers"
 	k8sfake "k8s.io/client-go/kubernetes/fake"
@@ -40,7 +37,7 @@ import (
 	apiregistrationv1 "k8s.io/kube-aggregator/pkg/apis/apiregistration/v1"
 	apiregistrationfake "k8s.io/kube-aggregator/pkg/client/clientset_generated/clientset/fake"
 
-	"github.com/operator-framework/operator-lifecycle-manager/pkg/api/apis/operators/v1alpha1"
+	"github.com/operator-framework/api/pkg/operators/v1alpha1"
 	"github.com/operator-framework/operator-lifecycle-manager/pkg/api/client/clientset/versioned/fake"
 	"github.com/operator-framework/operator-lifecycle-manager/pkg/api/client/informers/externalversions"
 	olmerrors "github.com/operator-framework/operator-lifecycle-manager/pkg/controller/errors"
@@ -49,7 +46,6 @@ import (
 	"github.com/operator-framework/operator-lifecycle-manager/pkg/controller/registry/resolver"
 	"github.com/operator-framework/operator-lifecycle-manager/pkg/fakes"
 	"github.com/operator-framework/operator-lifecycle-manager/pkg/lib/clientfake"
-	controllerclient "github.com/operator-framework/operator-lifecycle-manager/pkg/lib/controller-runtime/client"
 	"github.com/operator-framework/operator-lifecycle-manager/pkg/lib/operatorclient"
 	"github.com/operator-framework/operator-lifecycle-manager/pkg/lib/operatorlister"
 	"github.com/operator-framework/operator-lifecycle-manager/pkg/lib/ownerutil"
@@ -139,288 +135,55 @@ func TestTransitionInstallPlan(t *testing.T) {
 	}
 }
 
-func TestEnsureCRDVersions(t *testing.T) {
-	mainCRDPlural := "ins-main-abcde"
-
-	currentVersions := []v1beta1.CustomResourceDefinitionVersion{
-		{
-			Name:    "v1alpha1",
-			Served:  true,
-			Storage: true,
-		},
-	}
-
-	addedVersions := []v1beta1.CustomResourceDefinitionVersion{
-		{
-			Name:    "v1alpha1",
-			Served:  true,
-			Storage: false,
-		},
-		{
-			Name:    "v1alpha2",
-			Served:  true,
-			Storage: true,
-		},
-	}
-
-	missingVersions := []v1beta1.CustomResourceDefinitionVersion{
-		{
-			Name:    "v1alpha2",
-			Served:  true,
-			Storage: true,
-		},
-	}
+func TestSyncInstallPlanUnhappy(t *testing.T) {
+	namespace := "ns"
 
 	tests := []struct {
-		name            string
-		oldCRD          v1beta1.CustomResourceDefinition
-		newCRD          v1beta1.CustomResourceDefinition
-		expectedFailure bool
+		testName string
+		in       *v1alpha1.InstallPlan
+		err      error
 	}{
 		{
-			name: "existing versions are present",
-			oldCRD: func() v1beta1.CustomResourceDefinition {
-				oldCRD := crd(mainCRDPlural)
-				oldCRD.Spec.Version = ""
-				oldCRD.Spec.Versions = currentVersions
-				return oldCRD
-			}(),
-			newCRD: func() v1beta1.CustomResourceDefinition {
-				newCRD := crd(mainCRDPlural)
-				newCRD.Spec.Version = ""
-				newCRD.Spec.Versions = addedVersions
-				return newCRD
-			}(),
-			expectedFailure: false,
+			testName: "NoStatus",
+			in:       installPlan("p", namespace, v1alpha1.InstallPlanPhaseNone),
+			err:      nil,
 		},
 		{
-			name: "missing versions in new CRD",
-			oldCRD: func() v1beta1.CustomResourceDefinition {
-				oldCRD := crd(mainCRDPlural)
-				oldCRD.Spec.Version = ""
-				oldCRD.Spec.Versions = currentVersions
-				return oldCRD
-			}(),
-			newCRD: func() v1beta1.CustomResourceDefinition {
-				newCRD := crd(mainCRDPlural)
-				newCRD.Spec.Version = ""
-				newCRD.Spec.Versions = missingVersions
-				return newCRD
-			}(),
-			expectedFailure: true,
-		},
-		{
-			name: "missing version in new CRD",
-			oldCRD: func() v1beta1.CustomResourceDefinition {
-				oldCRD := crd(mainCRDPlural)
-				oldCRD.Spec.Version = "v1alpha1"
-				return oldCRD
-			}(),
-			newCRD: func() v1beta1.CustomResourceDefinition {
-				newCRD := crd(mainCRDPlural)
-				newCRD.Spec.Version = "v1alpha2"
-				return newCRD
-			}(),
-			expectedFailure: true,
-		},
-		{
-			name: "existing version is present in new CRD's versions",
-			oldCRD: func() v1beta1.CustomResourceDefinition {
-				oldCRD := crd(mainCRDPlural)
-				oldCRD.Spec.Version = "v1alpha1"
-				return oldCRD
-			}(),
-			newCRD: func() v1beta1.CustomResourceDefinition {
-				newCRD := crd(mainCRDPlural)
-				newCRD.Spec.Version = ""
-				newCRD.Spec.Versions = addedVersions
-				return newCRD
-			}(),
-			expectedFailure: false,
+			// This checks that installplans are not applied when no operatorgroup is present
+			testName: "HasSteps/NoOperatorGroup",
+			in: withSteps(installPlan("p", namespace, v1alpha1.InstallPlanPhaseInstalling, "csv"),
+				[]*v1alpha1.Step{
+					{
+						Resource: v1alpha1.StepResource{
+							CatalogSource:          "catalog",
+							CatalogSourceNamespace: namespace,
+							Group:                  "",
+							Version:                "v1",
+							Kind:                   "ServiceAccount",
+							Name:                   "sa",
+							Manifest: toManifest(t, serviceAccount("sa", namespace, "",
+								objectReference("init secret"))),
+						},
+						Status: v1alpha1.StepStatusUnknown,
+					},
+				},
+			),
+			err:  fmt.Errorf("attenuated service account query failed - no operator group found that is managing this namespace"),
 		},
 	}
 
 	for _, tt := range tests {
-		err := ensureCRDVersions(&tt.oldCRD, &tt.newCRD)
-		if tt.expectedFailure {
-			require.Error(t, err)
-		}
-	}
-}
+		t.Run(tt.testName, func(t *testing.T) {
+			ctx, cancel := context.WithCancel(context.TODO())
+			defer cancel()
 
-func TestRemoveDeprecatedStoredVersions(t *testing.T) {
-	mainCRDPlural := "ins-main-test"
-
-	currentVersions := []v1beta1.CustomResourceDefinitionVersion{
-		{
-			Name:    "v1alpha1",
-			Served:  true,
-			Storage: false,
-		},
-		{
-			Name:    "v1alpha2",
-			Served:  true,
-			Storage: true,
-		},
-	}
-
-	newVersions := []v1beta1.CustomResourceDefinitionVersion{
-		{
-			Name:    "v1alpha2",
-			Served:  true,
-			Storage: false,
-		},
-		{
-			Name:    "v1beta1",
-			Served:  true,
-			Storage: true,
-		},
-	}
-
-	crdStatusStoredVersions := v1beta1.CustomResourceDefinitionStatus{
-		StoredVersions: []string{},
-	}
-
-	tests := []struct {
-		name           string
-		oldCRD         v1beta1.CustomResourceDefinition
-		newCRD         v1beta1.CustomResourceDefinition
-		expectedResult []string
-	}{
-		{
-			name: "only one stored version exists",
-			oldCRD: func() v1beta1.CustomResourceDefinition {
-				oldCRD := crd(mainCRDPlural)
-				oldCRD.Spec.Version = ""
-				oldCRD.Spec.Versions = currentVersions
-				oldCRD.Status = crdStatusStoredVersions
-				oldCRD.Status.StoredVersions = []string{"v1alpha1"}
-				return oldCRD
-			}(),
-			newCRD: func() v1beta1.CustomResourceDefinition {
-				newCRD := crd(mainCRDPlural)
-				newCRD.Spec.Version = ""
-				newCRD.Spec.Versions = newVersions
-				return newCRD
-			}(),
-			expectedResult: nil,
-		},
-		{
-			name: "multiple stored versions with one deprecated version",
-			oldCRD: func() v1beta1.CustomResourceDefinition {
-				oldCRD := crd(mainCRDPlural)
-				oldCRD.Spec.Version = ""
-				oldCRD.Spec.Versions = currentVersions
-				oldCRD.Status.StoredVersions = []string{"v1alpha1", "v1alpha2"}
-				return oldCRD
-			}(),
-			newCRD: func() v1beta1.CustomResourceDefinition {
-				newCRD := crd(mainCRDPlural)
-				newCRD.Spec.Version = ""
-				newCRD.Spec.Versions = newVersions
-				return newCRD
-			}(),
-			expectedResult: []string{"v1alpha2"},
-		},
-		{
-			name: "multiple stored versions with all deprecated version",
-			oldCRD: func() v1beta1.CustomResourceDefinition {
-				oldCRD := crd(mainCRDPlural)
-				oldCRD.Spec.Version = ""
-				oldCRD.Spec.Versions = currentVersions
-				oldCRD.Status.StoredVersions = []string{"v1alpha1", "v1alpha3"}
-				return oldCRD
-			}(),
-			newCRD: func() v1beta1.CustomResourceDefinition {
-				newCRD := crd(mainCRDPlural)
-				newCRD.Spec.Version = ""
-				newCRD.Spec.Versions = newVersions
-				return newCRD
-			}(),
-			expectedResult: nil,
-		},
-	}
-
-	for _, tt := range tests {
-		resultCRD := removeDeprecatedStoredVersions(&tt.oldCRD, &tt.newCRD)
-		require.Equal(t, tt.expectedResult, resultCRD)
-	}
-}
-
-type ipSet []v1alpha1.InstallPlan
-
-func (ipSet) Generate(rand *rand.Rand, size int) reflect.Value {
-	ips := []v1alpha1.InstallPlan{}
-
-	// each i is the generation value
-	for i := 0; i < rand.Intn(size)+1; i++ {
-
-		// generate a few at each generation to account for bugs that don't increment the generation
-		for j := 0; j < rand.Intn(3); j++ {
-			ips = append(ips, v1alpha1.InstallPlan{
-				ObjectMeta: metav1.ObjectMeta{
-					Namespace: "ns",
-					Name:      names.SimpleNameGenerator.GenerateName(fmt.Sprintf("%d", i)),
-				},
-				Spec: v1alpha1.InstallPlanSpec{
-					Generation: i,
-				},
-			})
-		}
-	}
-	return reflect.ValueOf(ipSet(ips))
-}
-
-func TestGCInstallPlans(t *testing.T) {
-	f := func(ips ipSet) bool {
-		if len(ips) == 0 {
-			return true
-		}
-		ctx, cancel := context.WithCancel(context.TODO())
-		defer cancel()
-
-		var maxGen int64 = 0
-		for _, i := range ips {
-			if g := i.Generation; g > maxGen {
-				maxGen = g
-			}
-		}
-		objs := make([]runtime.Object, 0)
-		for _, i := range ips {
-			objs = append(objs, i.DeepCopy())
-		}
-		op, err := NewFakeOperator(ctx, "ns", []string{"ns"}, withClientObjs(objs...))
-		require.NoError(t, err)
-
-		out := make([]v1alpha1.InstallPlan, 0)
-		for {
-			op.gcInstallPlans(logrus.New(), "ns")
+			op, err := NewFakeOperator(ctx, namespace, []string{namespace}, withClientObjs(tt.in))
 			require.NoError(t, err)
 
-			outList, err := op.client.OperatorsV1alpha1().InstallPlans("ns").List(metav1.ListOptions{})
-			require.NoError(t, err)
-			out = outList.Items
-
-			if len(out) <= maxInstallPlanCount {
-				break
-			}
-		}
-
-		keptMax := false
-		for _, o := range out {
-			if o.Generation == maxGen {
-				keptMax = true
-				break
-			}
-		}
-		require.True(t, keptMax)
-
-		if len(ips) < maxInstallPlanCount {
-			return len(out) == len(ips)
-		}
-		return len(out) == maxInstallPlanCount
+			err = op.syncInstallPlans(tt.in)
+			require.Equal(t, tt.err, err)
+		})
 	}
-	require.NoError(t, quick.Check(f, nil))
 }
 
 func TestExecutePlan(t *testing.T) {
@@ -494,6 +257,69 @@ func TestExecutePlan(t *testing.T) {
 			want: []runtime.Object{serviceAccount("sa", namespace, "", objectReference("init secret"))},
 			err:  nil,
 		},
+		{
+			testName: "CreateConfigMap",
+			in: withSteps(installPlan("p", namespace, v1alpha1.InstallPlanPhaseInstalling, "csv"),
+				[]*v1alpha1.Step{
+					{
+						Resource: v1alpha1.StepResource{
+							CatalogSource:          "catalog",
+							CatalogSourceNamespace: namespace,
+							Group:                  "",
+							Version:                "v1",
+							Kind:                   "ConfigMap",
+							Name:                   "cfg",
+							Manifest:               toManifest(t, configMap("cfg", namespace)),
+						},
+						Status: v1alpha1.StepStatusUnknown,
+					},
+				},
+			),
+			want: []runtime.Object{configMap("cfg", namespace)},
+			err:  nil,
+		},
+		{
+			testName: "CreateSecretFromBundle",
+			in: withSteps(installPlan("p", namespace, v1alpha1.InstallPlanPhaseInstalling, "csv"),
+				[]*v1alpha1.Step{
+					{
+						Resource: v1alpha1.StepResource{
+							CatalogSource:          "catalog",
+							CatalogSourceNamespace: namespace,
+							Group:                  "",
+							Version:                "v1",
+							Kind:                   "BundleSecret",
+							Name:                   "s",
+							Manifest:               toManifest(t, secret("s", namespace)),
+						},
+						Status: v1alpha1.StepStatusUnknown,
+					},
+				},
+			),
+			want: []runtime.Object{secret("s", namespace)},
+			err:  nil,
+		},
+		{
+			testName: "DoesNotCreateSecretNotFromBundle",
+			in: withSteps(installPlan("p", namespace, v1alpha1.InstallPlanPhaseInstalling, "csv"),
+				[]*v1alpha1.Step{
+					{
+						Resource: v1alpha1.StepResource{
+							CatalogSource:          "catalog",
+							CatalogSourceNamespace: namespace,
+							Group:                  "",
+							Version:                "v1",
+							Kind:                   "Secret",
+							Name:                   "s",
+							Manifest:               toManifest(t, secret("s", namespace)),
+						},
+						Status: v1alpha1.StepStatusUnknown,
+					},
+				},
+			),
+			want: []runtime.Object{},
+			err:  fmt.Errorf("secret s does not exist - secrets \"s\" not found"),
+		},
 		{
 			testName: "UpdateServiceAccountWithSameFields",
 			in: withSteps(installPlan("p", namespace, v1alpha1.InstallPlanPhaseInstalling, "csv"),
@@ -594,7 +420,7 @@ func TestExecutePlan(t *testing.T) {
 					},
 				},
 			),
-			extObjs: []runtime.Object{decodeFile(t, "./testdata/prometheusrule.crd.yaml", &v1beta1.CustomResourceDefinition{})},
+			extObjs: []runtime.Object{decodeFile(t, "./testdata/prometheusrule.crd.yaml", &apiextensionsv1beta1.CustomResourceDefinition{})},
 			want: []runtime.Object{
 				csv("csv", namespace, nil, nil),
 				modify(t, decodeFile(t, "./testdata/prometheusrule.cr.yaml", &unstructured.Unstructured{}),
@@ -604,6 +430,41 @@ func TestExecutePlan(t *testing.T) {
 			},
 			err: nil,
 		},
+		{
+			testName: "V1CRDResourceIsCreated",
+			in: withSteps(installPlan("p", namespace, v1alpha1.InstallPlanPhaseInstalling, "crdv1"),
+				[]*v1alpha1.Step{
+					{
+						Resource: v1alpha1.StepResource{
+							CatalogSource:          "catalog",
+							CatalogSourceNamespace: namespace,
+							Group:                  "apiextensions.k8s.io",
+							Version:                "v1",
+							Kind:                   crdKind,
+							Name:                   "crd",
+							Manifest: toManifest(t,
+								&apiextensionsv1.CustomResourceDefinition{
+									TypeMeta: metav1.TypeMeta{
+										Kind:       "CustomResourceDefinition",
+										APIVersion: "apiextensions.k8s.io/v1", // v1 CRD version of API
+									},
+									ObjectMeta: metav1.ObjectMeta{Name: "test"},
+									Spec:       apiextensionsv1.CustomResourceDefinitionSpec{},
+								}),
+						},
+						Status: v1alpha1.StepStatusUnknown,
+					},
+				}),
+			want: []runtime.Object{
+				&apiextensionsv1.CustomResourceDefinition{
+					ObjectMeta: metav1.ObjectMeta{Name: "test"},
+					TypeMeta: metav1.TypeMeta{
+						Kind:       "CustomResourceDefinition",
+						APIVersion: "apiextensions.k8s.io/v1", // v1 CRD version of API
+					},
+				},
+			},
+		},
 	}
 
 	for _, tt := range tests {
@@ -638,10 +499,14 @@ func TestExecutePlan(t *testing.T) {
 					fetched, err = op.opClient.GetSecret(namespace, o.GetName())
 				case *corev1.Service:
 					fetched, err = op.opClient.GetService(namespace, o.GetName())
-				case *v1beta1.CustomResourceDefinition:
-					fetched, err = op.opClient.ApiextensionsV1beta1Interface().ApiextensionsV1beta1().CustomResourceDefinitions().Get(o.GetName(), getOpts)
+				case *corev1.ConfigMap:
+					fetched, err = op.opClient.GetConfigMap(namespace, o.GetName())
+				case *apiextensionsv1beta1.CustomResourceDefinition:
+					fetched, err = op.opClient.ApiextensionsInterface().ApiextensionsV1beta1().CustomResourceDefinitions().Get(context.TODO(), o.GetName(), getOpts)
+				case *apiextensionsv1.CustomResourceDefinition:
+					fetched, err = op.opClient.ApiextensionsInterface().ApiextensionsV1().CustomResourceDefinitions().Get(context.TODO(), o.GetName(), getOpts)
 				case *v1alpha1.ClusterServiceVersion:
-					fetched, err = op.client.OperatorsV1alpha1().ClusterServiceVersions(namespace).Get(o.GetName(), getOpts)
+					fetched, err = op.client.OperatorsV1alpha1().ClusterServiceVersions(namespace).Get(context.TODO(), o.GetName(), getOpts)
 				case *unstructured.Unstructured:
 					// Get the resource from the GVK
 					gvk := o.GroupVersionKind()
@@ -656,11 +521,11 @@ func TestExecutePlan(t *testing.T) {
 					}
 
 					if r.Namespaced {
-						fetched, err = op.dynamicClient.Resource(gvr).Namespace(namespace).Get(o.GetName(), getOpts)
+						fetched, err = op.dynamicClient.Resource(gvr).Namespace(namespace).Get(context.TODO(), o.GetName(), getOpts)
 						break
 					}
 
-					fetched, err = op.dynamicClient.Resource(gvr).Get(o.GetName(), getOpts)
+					fetched, err = op.dynamicClient.Resource(gvr).Get(context.TODO(), o.GetName(), getOpts)
 				default:
 					require.Failf(t, "couldn't find expected object", "%#v", obj)
 				}
@@ -1030,7 +895,7 @@ func TestSyncCatalogSources(t *testing.T) {
 			}
 
 			// Get updated catalog and check status
-			updated, err := op.client.OperatorsV1alpha1().CatalogSources(tt.catalogSource.GetNamespace()).Get(tt.catalogSource.GetName(), metav1.GetOptions{})
+			updated, err := op.client.OperatorsV1alpha1().CatalogSources(tt.catalogSource.GetNamespace()).Get(context.TODO(), tt.catalogSource.GetName(), metav1.GetOptions{})
 			require.NoError(t, err)
 			require.NotEmpty(t, updated)
 
@@ -1039,7 +904,7 @@ func TestSyncCatalogSources(t *testing.T) {
 				require.Equal(t, *tt.expectedStatus, updated.Status)
 
 				if tt.catalogSource.Spec.ConfigMap != "" {
-					configMap, err := op.opClient.KubernetesInterface().CoreV1().ConfigMaps(tt.catalogSource.GetNamespace()).Get(tt.catalogSource.Spec.ConfigMap, metav1.GetOptions{})
+					configMap, err := op.opClient.KubernetesInterface().CoreV1().ConfigMaps(tt.catalogSource.GetNamespace()).Get(context.TODO(), tt.catalogSource.Spec.ConfigMap, metav1.GetOptions{})
 					require.NoError(t, err)
 					require.True(t, ownerutil.EnsureOwner(configMap, updated))
 				}
@@ -1049,7 +914,7 @@ func TestSyncCatalogSources(t *testing.T) {
 				switch o.(type) {
 				case *corev1.Pod:
 					t.Log("verifying pod")
-					pods, err := op.opClient.KubernetesInterface().CoreV1().Pods(tt.catalogSource.Namespace).List(metav1.ListOptions{})
+					pods, err := op.opClient.KubernetesInterface().CoreV1().Pods(tt.catalogSource.Namespace).List(context.TODO(), metav1.ListOptions{})
 					require.NoError(t, err)
 					require.Len(t, pods.Items, 1)
 
@@ -1137,7 +1002,7 @@ func TestCompetingCRDOwnersExist(t *testing.T) {
 
 func fakeConfigMapData() map[string]string {
 	data := make(map[string]string)
-	yaml, err := yaml.Marshal([]v1beta1.CustomResourceDefinition{crd("fake-crd")})
+	yaml, err := yaml.Marshal([]apiextensionsv1beta1.CustomResourceDefinition{crd("fake-crd")})
 	if err != nil {
 		return data
 	}
@@ -1226,7 +1091,7 @@ func NewFakeOperator(ctx context.Context, namespace string, namespaces []string,
 	dynamicClientFake := fakedynamic.NewSimpleDynamicClient(runtime.NewScheme())
 
 	// Create operator namespace
-	_, err := opClientFake.KubernetesInterface().CoreV1().Namespaces().Create(&corev1.Namespace{ObjectMeta: metav1.ObjectMeta{Name: namespace}})
+	_, err := opClientFake.KubernetesInterface().CoreV1().Namespaces().Create(context.TODO(), &corev1.Namespace{ObjectMeta: metav1.ObjectMeta{Name: namespace}}, metav1.CreateOptions{})
 	if err != nil {
 		return nil, err
 	}
@@ -1236,7 +1101,7 @@ func NewFakeOperator(ctx context.Context, namespace string, namespaces []string,
 	var sharedInformers []cache.SharedIndexInformer
 	for _, ns := range namespaces {
 		if ns != namespace {
-			_, err := opClientFake.KubernetesInterface().CoreV1().Namespaces().Create(&corev1.Namespace{ObjectMeta: metav1.ObjectMeta{Name: ns}})
+			_, err := opClientFake.KubernetesInterface().CoreV1().Namespaces().Create(context.TODO(), &corev1.Namespace{ObjectMeta: metav1.ObjectMeta{Name: ns}}, metav1.CreateOptions{})
 			if err != nil {
 				return nil, err
 			}
@@ -1302,14 +1167,7 @@ func NewFakeOperator(ctx context.Context, namespace string, namespaces []string,
 	}
 	op.sources = grpc.NewSourceStore(config.logger, 1*time.Second, 5*time.Second, op.syncSourceState)
 	if op.reconciler == nil {
-		s := runtime.NewScheme()
-		err := k8sfake.AddToScheme(s)
-		if err != nil {
-			return nil, err
-		}
-		applier := controllerclient.NewFakeApplier(s, "testowner")
-
-		op.reconciler = reconciler.NewRegistryReconcilerFactory(lister, op.opClient, "test:pod", op.now, applier)
+		op.reconciler = reconciler.NewRegistryReconcilerFactory(lister, op.opClient, "test:pod", op.now)
 	}
 
 	op.RunInformers(ctx)
@@ -1361,6 +1219,10 @@ func csv(name, namespace string, owned, required []string) *v1alpha1.ClusterServ
 	}
 
 	return &v1alpha1.ClusterServiceVersion{
+		TypeMeta: metav1.TypeMeta{
+			Kind:       csvKind,
+			APIVersion: "operators.coreos.com/v1alpha1",
+		},
 		ObjectMeta: metav1.ObjectMeta{
 			Name:      name,
 			Namespace: namespace,
@@ -1374,15 +1236,35 @@ func csv(name, namespace string, owned, required []string) *v1alpha1.ClusterServ
 	}
 }
 
-func crd(name string) v1beta1.CustomResourceDefinition {
-	return v1beta1.CustomResourceDefinition{
+func crd(name string) apiextensionsv1beta1.CustomResourceDefinition {
+	return apiextensionsv1beta1.CustomResourceDefinition{
 		ObjectMeta: metav1.ObjectMeta{
 			Name: name,
 		},
-		Spec: v1beta1.CustomResourceDefinitionSpec{
+		Spec: apiextensionsv1beta1.CustomResourceDefinitionSpec{
 			Group:   name + "group",
 			Version: "v1",
-			Names: v1beta1.CustomResourceDefinitionNames{
+			Names: apiextensionsv1beta1.CustomResourceDefinitionNames{
+				Kind: name,
+			},
+		},
+	}
+}
+
+func v1crd(name string) apiextensionsv1.CustomResourceDefinition {
+	return apiextensionsv1.CustomResourceDefinition{
+		ObjectMeta: metav1.ObjectMeta{
+			Name: name,
+		},
+		Spec: apiextensionsv1.CustomResourceDefinitionSpec{
+			Group: name + "group",
+			Versions: []apiextensionsv1.CustomResourceDefinitionVersion{
+				{
+					Name:   "v1",
+					Served: true,
+				},
+			},
+			Names: apiextensionsv1.CustomResourceDefinitionNames{
 				Kind: name,
 			},
 		},
@@ -1391,6 +1273,19 @@ func crd(name string) v1beta1.CustomResourceDefinition {
 
 func service(name, namespace string) *corev1.Service {
 	return &corev1.Service{
+		TypeMeta: metav1.TypeMeta{
+			Kind:       serviceKind,
+			APIVersion: "",
+		},
+		ObjectMeta: metav1.ObjectMeta{
+			Name:      name,
+			Namespace: namespace,
+		},
+	}
+}
+
+func secret(name, namespace string) *corev1.Secret {
+	return &corev1.Secret{
 		ObjectMeta: metav1.ObjectMeta{
 			Name:      name,
 			Namespace: namespace,
@@ -1401,15 +1296,24 @@ func service(name, namespace string) *corev1.Service {
 func serviceAccount(name, namespace, generateName string, secretRef *corev1.ObjectReference) *corev1.ServiceAccount {
 	if secretRef == nil {
 		return &corev1.ServiceAccount{
+			TypeMeta:   metav1.TypeMeta{Kind: serviceAccountKind, APIVersion: ""},
 			ObjectMeta: metav1.ObjectMeta{Name: name, Namespace: namespace, GenerateName: generateName},
 		}
 	}
 	return &corev1.ServiceAccount{
+		TypeMeta:   metav1.TypeMeta{Kind: serviceAccountKind, APIVersion: ""},
 		ObjectMeta: metav1.ObjectMeta{Name: name, Namespace: namespace, GenerateName: generateName},
 		Secrets:    []corev1.ObjectReference{*secretRef},
 	}
 }
 
+func configMap(name, namespace string) *corev1.ConfigMap {
+	return &corev1.ConfigMap{
+		TypeMeta:   metav1.TypeMeta{Kind: configMapKind},
+		ObjectMeta: metav1.ObjectMeta{Name: name, Namespace: namespace},
+	}
+}
+
 func objectReference(name string) *corev1.ObjectReference {
 	if name == "" {
 		return &corev1.ObjectReference{}
@@ -1510,15 +1414,15 @@ func apiResourcesForObjects(objs []runtime.Object) []*metav1.APIResourceList {
 	apis := []*metav1.APIResourceList{}
 	for _, o := range objs {
 		switch o.(type) {
-		case *v1beta1.CustomResourceDefinition:
-			crd := o.(*v1beta1.CustomResourceDefinition)
+		case *apiextensionsv1beta1.CustomResourceDefinition:
+			crd := o.(*apiextensionsv1beta1.CustomResourceDefinition)
 			apis = append(apis, &metav1.APIResourceList{
 				GroupVersion: metav1.GroupVersion{Group: crd.Spec.Group, Version: crd.Spec.Versions[0].Name}.String(),
 				APIResources: []metav1.APIResource{
 					{
 						Name:         crd.GetName(),
 						SingularName: crd.Spec.Names.Singular,
-						Namespaced:   crd.Spec.Scope == v1beta1.NamespaceScoped,
+						Namespaced:   crd.Spec.Scope == apiextensionsv1beta1.NamespaceScoped,
 						Group:        crd.Spec.Group,
 						Version:      crd.Spec.Versions[0].Name,
 						Kind:         crd.Spec.Names.Kind,
